function [tsave,ysave] = RKF45Simple(f,t0,T,y0,h0,atol,rtol,varargin)

% function RKF45Simple
%
% Integriere die gewoehnliche Diff.-Gleichung erster Ordnung
%
%   dy/dt = f(t,y(t))
%
% mit einem (zu?) einfachen Runge-Kutta-Fehlberg Verfahren basierend auf
% eingebetteten Runge-Kutta Verfahren der Ordnung 4 und 5.
%
% Parameters: f        ... rechte Seite f(t,y(t)) der gew. Diff.-Gl.
%             t0,T     ... Start- und End-Zeit
%             y0       ... Anfangswert
%             h0       ... Anfangs-Schrittweite
%             atol     ... Absolute Toleranz
%             varargin ... Fuer Infos zu varargin, s. die MATLAB
%                          Dokumentation
%
% Returns: tsave ... Zeiten
%          ysave ... approx. Loesung zu Zeiten t
%
% Notes: Diese Implementierung hat folgende Schwaechen:
%
%          - Wenn die Schrittweite einmal verkleinert wurde, z.B. wenn die
%            Loesung stark variiert, wird sie nicht mehr erhoeht, z.B.
%            wenn die Loesung weniger variiert.
%          - Es koennte passieren, dass der Algorithmus die Schrittweite
%            halbiert ohne jemals das Toleranz-Kriterium zu erreichen, z.B.
%            wenn die Toleranzen sehr kleint gewaehlt sind.
%

% refinement_case : parameter for testing
%  0 : adaptive method
%  1 : RK5 with uniform mesh
%  2 : RK4 with uniform mesh
if (nargin == 8)
  refinement_case = varargin{1};
else
  refinement_case = 0;
end

% Initialisiere Zeit- und Loesungs-Vektor
y = y0;
t = t0;
h = h0;

tsave = t0;
ysave = y0;

while t < T
    k1 = f(t,y);
    k2 = f(t + h/4,y + h*k1/4);
    k3 = f(t + 3*h/8,y + h*(3*k1 + 9*k2)/32);
    k4 = f(t + 12*h/13,y + h*(1932*k1 - 7200*k2 + 7296*k3)/2197);
    k5 = f(t + h,y + h*(439*k1/216 -8*k2 +3680*k3/513 - 845*k4/4104));
    k6 = f(t + h/2,y + h*(-8*k1/27 + 2*k2 -3544*k3/2565+1859*k4/4104 - 11*k5/40));

    ytemp_4 = y + h*(25*k1/216 + 1408*k3/2565 + 2197*k4/4104 - 1*k5/5);
    ytemp_5 = y + h*(16*k1/135 + 6656*k3/12825 + 28561*k4/56430 - 9*k5/50 + 2*k6/55);

    if (refinement_case == 0)
        if norm(ytemp_5 - ytemp_4) < atol + norm(y)*rtol
          t = t + h;
          y = ytemp_5;
          h = min(h, T - t);

          tsave = [tsave, t];
          ysave = [ysave, y];
        else
          h = h/2;
        end
    elseif (refinement_case == 1)
      t     = t + h;
      y     = ytemp_5;
      tsave = [tsave, t];
      ysave = [ysave, y];
      h     = min(h, T - t);
    elseif (refinement_case == 2)
      t     = t + h;
      y     = ytemp_4;
      tsave = [tsave, t];
      ysave = [ysave, y];
      h     = min(h, T - t);
    end
end
