function [t,y] = adaptHeunSimple(f,t0,T,y0,h0,atol,rtol)

% function adaptHeunSimple
%
% Integriere die gewoehnliche Diff.-Gleichung erster Ordnung
%
%   dy/dt = f(t,y(t))
%
% mit einem (zu?) einfachen adaptiven Heun Verfahren mit
% Schrittweitenhalbierung-Fehlerschaetzer.
%
% Parameters: f    ... rechte Seite f(t,y(t)) der gew. Diff.-Gl.
%             t0,T ... Start- und End-Zeit
%             y0   ... Anfangswert
%             h0   ... Anfangs-Schrittweite
%             atol ... Absolute Toleranz
%             rtol ... Relative Toleranz
%
% Returns: t ... Zeiten
%          y ... approx. Loesung zu Zeiten t
%
% Notes: Diese Implementierung hat folgende Schwaechen:
%
%          - Wenn die Schrittweite einmal verkleinert wurde, z.B. wenn die
%            Loesung stark variiert, wird sie nicht mehr erhoeht, z.B.
%            wenn die Loesung weniger variiert.
%          - Es koennte passieren, dass der Algorithmus die Schrittweite
%            halbiert ohne jemals das Toleranz-Kriterium zu erreichen, z.B.
%            wenn die Toleranzen sehr kleint gewaehlt sind.

% Parameter
  p = 2; % KO des Heun Verfahren

% Initialisiere Zeit- und Loesungs-Vektor
  t = t0;
  y = y0;

% Mache Zeitschritte solange Endzeit nicht erreicht
  hj = h0; % Initialisiere Schrittweite
  while (t(end) < T)

    % Zwischenspeicherung der Daten zum j-ten Schritt
    tj = t(end);
    yj = y(:,end);

    % Korrigiere Schrittweite hj so, dass die Endzeit T genau erreicht wird
    hj = min(T - tj,hj);

    % Heun Verfahren mit vollem Schritt h   --> y_{j+1} == yjp
    k1 = f(tj,yj);
    k2 = f(tj + hj,yj + hj*k1);
    yjp = yj +  0.5*hj*(k1 + k2);         % y_{j+1}
    % Heun Verfahren mit halbem Schritt h/2 --> yhut_{j+1} == yhutjp
    % Erster halb-Schritt
    % k1 = f(tj,yj); % koennen wir von oben uebernehmen!
                     % (damit sparen wir uns eine f Auswerung :-)
    k2 = f(tj + 0.5*hj,yj + 0.5*hj*k1);
    yhutjph = yj +  0.25*hj*(k1 + k2);    % yhut_{j+1/2} = yhutjph
    % Zweiter halb-Schritt
    k1 = f(tj + 0.5*hj,yhutjph);
    k2 = f(tj + hj,yhutjph + 0.5*hj*k1);
    yhutjp = yhutjph + 0.25*hj*(k1 + k2); % yhut_{j+1} = yhutjp

    % Berechne Fehlerschaetzer (fuer yhut_{j+1} = yhutjp, d.h. das genauere
    %                           Verfahren!)
    epsilonhutjp = norm(yhutjp - yjp)/(2^p - 1.);

    % Kontrolliere Toleranz-Kriterium
    if ( epsilonhutjp < atol + norm(yj)*rtol ) % akzeptiere Zeitschritt!
      t = [t,tj+hj];
      y = [y,yhutjp];
    else % verwerfe Zeitschritt und halbiere Schrittweite
      hj = 0.5*hj;
    end

  end
